<?php

namespace Highwaypro\App\Data\Model\Preferences;

use HighWayPro\Original\Collections\Collection;
use HighwayPro\App\HighwayPro\HTTP\Responses\Redirection;
use HighwayPro\Original\Collections\JSONMapper;
use HighwayPro\Original\Collections\Mapper\Mappable;
use HighwayPro\Original\Collections\Mapper\Types;
use Highwaypro\App\Data\Model\Preferences\Preferences;

Class UrlPreferences extends Preferences
{
    const CHARACTER_ALPHABETIC   = 'alphabetical';
    const CHARACTER_NUMERIC      = 'numeric';
    const CHARACTER_ALPHANUMERIC = 'alphanumeric';

    const CASE_ANY       = 'any';
    const CASE_UPPERCASE = 'uppercase';
    const CASE_LOWERCASE = 'lowercase';

    const REFERER_TYPE_ALWAYS = 'unsafe-url';
    const REFERER_TYPE_ORIGIN   = 'origin';
    const REFERER_TYPE_HIDE  = 'no-referrer';

    static public function fields()
    {
        return new Collection([
        'generateUrlsOnNewPosts_isEnabled'  => Types::BOOLEAN()->withDefault(true),
        'postUrl_UrlType'                   => Types::INTEGER(), // the base path of post urls
        'autoGeneratedPathType'             => Types::STRING()->withDefault(self::CHARACTER_NUMERIC) 
                                               ->allowed([
                                                    self::CHARACTER_ALPHABETIC, 
                                                    self::CHARACTER_NUMERIC, 
                                                    self::CHARACTER_ALPHANUMERIC
                                                ]),
        'autoGeneratedPathAlphanumericCase' => Types::STRING()->withDefault(self::CASE_ANY) 
                                               ->allowed([
                                                    self::CASE_ANY, 
                                                    self::CASE_UPPERCASE, 
                                                    self::CASE_LOWERCASE
                                                ]),
        'autoGeneratedPathLength'           => Types::INTEGER()->withDefault(6)->minimum(2), 
        'http_redirection_status_code'      => Types::INTEGER()->withDefault(Redirection::DEFAULT_STATUS_CODE)
                                                               ->minimum(Redirection::DEFAULT_STATUS_CODE_MINIMUM)
                                                               ->allowed([
                                                                    'Multiple Choice' => 300,
                                                                    'Moved Permanently' => 301,
                                                                    'Found' => 302,
                                                                    'See Other' => 303,
                                                                    'Not Modified' => 304,
                                                                    'Temporary Redirect' => 307,
                                                                    'Permanent Redirect' => 308
                                                                ]),
        'http_referrer_type'                => Types::STRING()->withDefault(self::REFERER_TYPE_ALWAYS) 
                                                              ->allowed([
                                                                   self::REFERER_TYPE_ALWAYS, 
                                                                   self::REFERER_TYPE_ORIGIN, 
                                                                   self::REFERER_TYPE_HIDE
                                                               ]),
        'http_referrer_custom_url'          => Types::STRING()
        ]);   
    }

    protected function getMap()
    {
        return static::fields()->asArray();
    }

    public function __construct($data)
    {
        $this->preferences = $this->map($data);
    }
}