<?php

namespace HighWayPro\App\HighWayPro\DestinationComponents\BuiltIn\Targets;

use HighWayPro\App\HighWayPro\DestinationComponents\BuiltIn\Targets\Errors\ContentTargetErrors;
use HighWayPro\App\HighWayPro\DestinationComponents\DestinationTargetComponent;
use HighWayPro\Original\Collections\Mapper\Types;
use HighWayPro\Original\Environment\Env;
use HighwayPro\App\HighwayPro\HTTP\Responses\PostTypeContentResponse;

/**
 * Displays the content of a post type in-situ (in the same URL), without redirecting to its regular perma-link.
 * As can be seen in ContentTarget::event, this Target runs in the template_redirect action.
 */
Class ContentTarget extends DestinationTargetComponent
{
    const event = 'template_redirect';

    const name  = 'highwaypro.ContentTarget';

    public static function title()
    {
        return __('Site Content', Env::textDomain());
    }

    public static function shortDescription()
    {
        return __('Show the contents of a post type, without a redirection.', Env::textDomain());
    }

    public static function description()
    {
        return __("Displays the content of a valid post type (Posts, Pages, Products, etc.) without redirecting the user.\n", Env::textDomain());
    }

    protected function parametersMap()
    {
        return [
            'id' => Types::INTEGER, // the post ID,
            'showCanonicalProperty' => Types::BOOLEAN()->withDefault(false)
        ];
    }

    public function response()
    {
        return PostTypeContentResponse::buildOrFailWithA404([
            'id' => $this->parameters->id,
            'showCanonicalProperty' => $this->parameters->showCanonicalProperty
        ]);
    }

    public function validateParameters()
    {
        if (get_post_status($this->parameters->id) === false) {
            return ContentTargetErrors::get('invalid_post_id');
        } elseif (!is_bool($this->parameters->rawDataFound->get('showCanonicalProperty'))) {
            return ContentTargetErrors::get('invalid_showCannonicalProperty_type');
        }

        return true;
    }
}