<?php

namespace HighWayPro\App\HighWayPro\DestinationComponents\BuiltIn\Targets;

use HighWayPro\App\HighWayPro\DestinationComponents\DestinationTargetComponent;
use HighWayPro\App\HighWayPro\Validators\Url\Errors\DestinationTargetErrors;
use HighWayPro\Original\Collections\Collection;
use HighWayPro\Original\Collections\Mapper\Types;
use HighWayPro\Original\Data\Drivers\WordPressDatabaseDriver;
use HighWayPro\Original\Environment\Env;
use HighwayPro\App\Data\Model\DestinationTargets\DestinationTarget;
use HighwayPro\App\Data\Model\DestinationTargets\DestinationTargetGateway;
use HighwayPro\App\Data\Model\Destinations\DestinationComponentDomain;
use HighwayPro\App\HighwayPro\HTTP\Responses\Redirection;

/*
    Redirects to one or more targets
*/
Class RotatingTarget extends DestinationTargetComponent
{
    const name  = 'highwaypro.RotatingTarget';
    protected $defaultURLPosition = 1;

    public static function title()
    {
        return __('Rotating URLs', Env::textDomain());
    }

    public static function shortDescription()
    {
        return __('Redirect to multiple URLs.', Env::textDomain());
    }

    public static function description()
    {
        return __("Redirect to multiple URLs, one at a time, using the same short URL. For example, if you set 3 different URLs, the first click will redirect to the first URL, the second click to the second URL, the third click to the third URL. This process starts over after the last URL from the list has been accessed.\n", Env::textDomain());
    }

    protected function parametersMap()
    {
        return [
            'urls' => Types::COLLECTION(),
            'currentURLPosition' => Types::INTEGER()->withDefault($this->defaultURLPosition)
        ];
    }

    public function response()
    {
        return (new Redirection)->to($this->getUrl($this->getUrlWithScheme($this->pickCurrentUrl())));
    }

    protected function pickCurrentUrl()
    {
        return $this->parameters->urls->atPosition($this->parameters->currentURLPosition);
    }
    
    public function incrementPointer()
    {
        (boolean) $pointerPointsToTheLastItem = $this->parameters->urls->count() === $this->parameters->currentURLPosition;

        if ($pointerPointsToTheLastItem) {
            $this->parameters->currentURLPosition = $this->defaultURLPosition;
        } else {
            $this->parameters->currentURLPosition++;
        }
    }

    /**
     * increments the index of the current url and updates the Destination target from the database
     * @param  DestinationComponentDomain $destinationComponentDomain
     */
    public function onShutDown(DestinationComponentDomain $destinationTarget)
    {
        $this->incrementPointer();

        $destinationTarget->parameters = json_encode($this->parameters);

        $destinationTarget->save();
    }
    
    public function validateParameters()
    {
        if ($this->parameters->urls->haveNone()) {
            return DestinationTargetErrors::get('no_target_url');
        } 

        return true;
    }
}