<?php

namespace HighwayPro\App\HighwayPro\HTTP\Responses;

use Exception;
use HighWayPro\App\Compatibility\YoastPluginCompatibility;
use HighWayPro\App\HighWayPro\HTTP\Response;
use HighWayPro\Original\Compatibility\BuiltIn\DefaultCompatibilty;
use HighWayPro\Original\Compatibility\BuiltIn\GlobalCompatibility;
use HighWayPro\Original\Compatibility\PluginCompatibility;
use HighwayPro\App\HighwayPro\HTTP\Responses\NotFoundResponse;
use WP_Query;

Class PostTypeContentResponse extends Response
{
    public $statusCode = 200;
    protected $needsToExit = false;

    protected $shouldSaveView = true;
    protected $shouldPrintBody = false;

    public static function buildOrFailWithA404(Array $data = [])
    {
        try {
            (object) $response = new static($data);
        } catch (Exception $exception) {
            (object) $response = new NotFoundResponse;
        }

        return $response;
    }

    public function __construct(Array $data = [])
    {
        parent::__construct($data);

        $this->id = isset($data['id'])? $data['id'] : null;
        $this->showCanonicalProperty = isset($data['showCanonicalProperty'])? $data['showCanonicalProperty'] : null;

        $this->postQuery = new WP_Query([
            'p' => $this->id,
            'post_type' => 'any'
        ]);

        if (!$this->postQuery->have_posts()) {
            throw new Exception('Post (Type) not found');
        }
    }
    
    /**
     * This needs to be run in the template_redirect action before being used.
     */
    protected function beforeSend()
    {
        global $wp_query, $post;

        /*WP_Post|null*/ $currentPost = (isset($this->postQuery->posts[0])?$this->postQuery->posts[0]:null);

        $this->postQuery->post = $currentPost;
        $wp_query = $this->postQuery;
        $wp_the_query = $this->postQuery;

        wp_reset_postdata();
        setup_postdata($currentPost);

        $this->registerEventToDefineCanonicalTag();
    }

    protected function registerEventToDefineCanonicalTag()
    {
        PluginCompatibility::handle([
            new GlobalCompatibility(function(){
                // gets executed everytime, regardless of the plugins installed.
                // in this case, let ourselves handle the adding of the canoncial tag
                remove_action('wp_head', 'rel_canonical');
            }),
            new YoastPluginCompatibility(function(){
                add_filter('wpseo_canonical', function($canonicalUrl) {
                    return !$this->showCanonicalProperty? false : $canonicalUrl;
                });
            }),
            new DefaultCompatibilty(function(){
                add_action('wp_head', function() {
                    if ($this->showCanonicalProperty) { 
                        (object) $postUrl = get_the_permalink($this->id);

                        print "<link rel=\"canonical\" href=\"{$postUrl}\" />";
                    }
                });
            })
        ]);
    }
}