import React, {Component} from 'react';

import { Observer } from '../../../domain/behaviour/events/observer/Observer';
import { UrlViewsFinder } from '../../../domain/data/finders/UrlViewsFinder';
import Notifications from '../../notifications/Notifications';
import PeriodOfTime from '../../../domain/date/PeriodOfTime';
import UrlStatistics from './Urls/statistics/UrlStatistics';

export default class OverviewPanelWindow extends Component
{
    state = {
        hasLoadedAnalytics: false,
        analytics: {},
        layout: 'default',
        periodOfTime: new PeriodOfTime,
        isRealoading: false
    };

    componentDidMount() 
    {
        this.setEvents();
        this.loadAnalytics();
        this.updateLayout();
    }

    setEvents() 
    {
        // when the period of time changes, we'll update the conponent.
        this.state.periodOfTime.add(new Observer(() => {
            this.setState({periodOfTime: this.state.periodOfTime}, this.loadAnalytics())
        }));

        window.addEventListener('resize', this.updateLayout.bind(this))
    }

    loadAnalytics() 
    {
        const urlViewsFinder = new UrlViewsFinder;

        this.setState({
            isRealoading: this.state.hasLoadedAnalytics? true : false,
        }, () => {
            urlViewsFinder.getStatsForAllUrls(this.state.periodOfTime).then(analytics => {
                if (analytics.type === 'all_url_views_read_sucess') {
                    this.setState({
                        hasLoadedAnalytics: true,
                        isRealoading: false,
                        analytics: analytics
                    })
                } else {
                    Notifications.addFromResponse({
                        message: 'Error loading analytics for all urls.',
                        state: 'error'
                    });
                }
            })
        })
    }

    render() 
    {
        return <UrlStatistics 
                    layout={this.state.layout} 
                    hasLoaded={this.state.hasLoadedAnalytics} 
                    getData={() => this.state.analytics}
                    periodOfTime={this.state.periodOfTime}
                    isRealoading={this.state.isRealoading}
               />
    }

    updateLayout() 
    {
        const newLayout = window.innerWidth < 1200? 'compact' : 'default';

        if (this.state.layout !== newLayout) {
            this.setState({
                layout: newLayout
            });
        }
    }
}