import './PresetsMenu.css';

import { DatePicker, MuiPickersUtilsProvider } from 'material-ui-pickers';
import ChevronLeftRounded from '@material-ui/icons/ChevronLeftRounded';
import ChevronRightRounded from '@material-ui/icons/ChevronRightRounded';
import DateFnsUtils from 'material-ui-pickers/utils/date-fns-utils';
import DateRangePicker from '@wojtekmaj/react-daterange-picker';
import Divider from '@material-ui/core/Divider';
import ListItem from '@material-ui/core/ListItem';
import ListItemText from '@material-ui/core/ListItemText';
import MenuItem from '@material-ui/core/MenuItem';
import React, {Component} from 'react';
import Select from '@material-ui/core/Select';
import format from 'date-fns/format';

import { HighWayPro } from '../../../../../domain/highwaypro/HighWayPro';
import { Strings } from '../../../../../domain/utilities/Strings';
import PeriodOfTime from '../../../../../domain/date/PeriodOfTime';
import ReadOnlyStateBar from './ReadOnlyStateBar';

class PresetsMenu extends Component {
    state = {
        presetMenuIsOpen: false,
        datePickerIsOpen: false
    };

    render() {
        return (
        <React.Fragment>
            <Select
                open={this.state.presetMenuIsOpen}
                onOpen={this.togglePresetMenu.bind(this, true)}
                onClose={this.togglePresetMenu.bind(this, false)}
                className="StatStateBarMenu StatStateBarPreset"
                classes={{root:'MuiInputBase-input MuiInput-underline hp-select'}}
                value={this.props.preset}
                onChange={({target}) => this.props.onChange(target.value)}
                renderValue={preset => this.isCustomRange(preset)? this.getCustomRangeText() : HighWayPro.analytics.presets[preset] || preset}
                >
                {this.getAllowedPresets().map(preset => (
                    this.isCustomRange(preset)? (
                        <React.Fragment>
                            <Divider />
                            <ListItem button onClick={() => {
                                this.togglePresetMenu(false, () => {
                                    window.setTimeout(this.togglePicker.bind(this, true), 400)
                                });
                            }}>
                                <ListItemText primary={'Custom range'} />
                            </ListItem>
                        </React.Fragment>
                    ) : (
                        <MenuItem value={preset}>{this.getReadablePreset(preset)}</MenuItem>
                    )
                ))}
            </Select>
            {this.props.customRangeIsEnabled && (
                <DateRangePicker
                    onChange={this.props.onDateRangeChange}
                    value={''}
                    isOpen={this.state.datePickerIsOpen}
                    onCalendarOpen={() => !this.state.datePickerIsOpen? this.togglePicker(true) : true}
                    onCalendarClose={() => this.togglePicker(false)}
                />
            )}
        </React.Fragment>
        );
    }

    isCustomRange(preset) 
    {
        return preset.toLowerCase() === 'customrange';
    }

    getAllowedPresets() 
    {
        const allPresets = Object.keys(HighWayPro.analytics.presets);

        if (!this.props.customRangeIsEnabled) {
            allPresets.splice(allPresets.findIndex(preset => this.isCustomRange(preset)), 1);
        }

        return allPresets;
    }

    getReadablePreset(preset) 
    {
        return (new PeriodOfTime).getReadablePreset(preset)
    }

    togglePresetMenu(isOpen, then) 
    {
        then = typeof then !== 'function'? (() => {}) : then;
        this.setState({
            presetMenuIsOpen: isOpen
        }, then)
    }

    togglePicker(isOpened) 
    {
        this.setState({datePickerIsOpen: isOpened})
    }

    getCustomRangeText() 
    {
        if (this.isCustomRange(this.props.preset)) {
            const APIRange = {
                type: 'remote',
                from: format(this.props.customRange.remote.from, 'YYYY-MM-dd'),
                to: format(this.props.customRange.remote.to, 'YYYY-MM-dd')
            }

            const periodOfTimeRange = {
                type: 'local',
                from: format(this.props.customRange.local.from, 'YYYY-MM-dd'),
                to: format(this.props.customRange.local.to, 'YYYY-MM-dd')
            }

            let rangetoUse = APIRange.from === periodOfTimeRange.from? APIRange : periodOfTimeRange;

            let range = (new PeriodOfTime).getReadableRange(rangetoUse);

            const rangeSource = rangetoUse.type === 'local'? 'L' : 'R';

            return (<span>{range} {<span class="hp-range-source">{rangeSource}</span>}</span>);
        }
    }
}

export default PresetsMenu;