import './StatStateBar.css';

import { DatePicker, MuiPickersUtilsProvider } from 'material-ui-pickers';
import ChevronLeftRounded from '@material-ui/icons/ChevronLeftRounded';
import ChevronRightRounded from '@material-ui/icons/ChevronRightRounded';
import DateFnsUtils from 'material-ui-pickers/utils/date-fns-utils';
import Divider from '@material-ui/core/Divider';
import ListItem from '@material-ui/core/ListItem';
import ListItemText from '@material-ui/core/ListItemText';
import MenuItem from '@material-ui/core/MenuItem';
import React, {Component} from 'react';
import Select from '@material-ui/core/Select';
import format from 'date-fns/format';

import { HighWayPro } from '../../../../../domain/highwaypro/HighWayPro';
import { Strings } from '../../../../../domain/utilities/Strings';
import PresetsMenu from './PresetsMenu';
import ReadOnlyStateBar from './ReadOnlyStateBar';
import UnitsMenu from './UnitsMenu';

class StatStateBar extends Component {
    state = {
        presetMenuIsOpen: false,
        datePickerIsOpen: false
    };

    render() {
        return (
            <div className="hp-StatStateBar">
                <div className="stateBarMenus">
                    <PresetsMenu 
                        customRangeIsEnabled={true} 
                        onChange={preset => this.props.periodOfTime.setProperty({preset})}
                        preset={this.props.periodOfTime.preset}
                        onDateRangeChange={([from, to]) => {this.props.periodOfTime.setProperty({
                            preset: 'customrange',
                            customRange: {
                                from: format(from, 'YYYY-MM-dd'),
                                to: format(to, 'YYYY-MM-dd')
                            }
                            })}
                        }
                        customRange={{
                            local: this.props.periodOfTime.customRange,
                            remote: this.props.statistics.inPeriod.periodCount.date
                        }}
                    />
                    <UnitsMenu 
                        unit={this.props.periodOfTime.unit}
                        preset={this.props.periodOfTime.preset}
                        onChange={unit => this.props.periodOfTime.setProperty({unit})}
                    />
                </div>
                <ReadOnlyStateBar 
                    periodOfTime={this.props.periodOfTime} 
                    date={this.props.statistics.inPeriod.periodCount.date}
                />
            </div>
        );
    }
}

export default StatStateBar;