import './UnitsMenu.css';

import { DatePicker, MuiPickersUtilsProvider } from 'material-ui-pickers';
import ChevronLeftRounded from '@material-ui/icons/ChevronLeftRounded';
import ChevronRightRounded from '@material-ui/icons/ChevronRightRounded';
import DateFnsUtils from 'material-ui-pickers/utils/date-fns-utils';
import DateRangePicker from '@wojtekmaj/react-daterange-picker';
import Divider from '@material-ui/core/Divider';
import ListItem from '@material-ui/core/ListItem';
import ListItemText from '@material-ui/core/ListItemText';
import MenuItem from '@material-ui/core/MenuItem';
import React, {Component} from 'react';
import Select from '@material-ui/core/Select';
import format from 'date-fns/format';

import { HighWayPro } from '../../../../../domain/highwaypro/HighWayPro';
import { Strings } from '../../../../../domain/utilities/Strings';
import PeriodOfTime from '../../../../../domain/date/PeriodOfTime';
import ReadOnlyStateBar from './ReadOnlyStateBar';

class UnitsMenu extends Component {
    render() {
        return (
            <Select
                className="StatStateBarMenu StatStateBarUnit"
                classes={{root:'MuiInputBase-input MuiInput-underline hp-select'}}
                value={this.props.unit}
                onChange={({target}) => this.props.onChange(target.value)}
                >
                {this.getAvailableUnitsForCurrentPreset().map(unit => (
                    <MenuItem value={unit}>{Strings.ucfirst(unit)}</MenuItem>
                ))}
            </Select>
        );
    }

    getAvailableUnitsForCurrentPreset() 
    {
        const units = HighWayPro.analytics.units.slice(); // let's clone it...

        if (!['today', 'lastday', 'twodaysago'].includes(this.props.preset.toLowerCase())) {
            units.splice(units.findIndex(unit => unit.toLowerCase() === 'hours'), 1);
        }

        return units;
    }
}

export default UnitsMenu;