import './UrlStatistics.css';

import InsertChartRounded from '@material-ui/icons/InsertChartRounded';
import React, {Component} from 'react';

import { HighWayPro } from '../../../../../domain/highwaypro/HighWayPro';
import { Strings } from '../../../../../domain/utilities/Strings';
import IconWithText from '../../../../icons/IconWithText';
import LoadingOverlay from './LoadingOverlay';
import UrlDestinationsLoading from
  '../UrlsContent/Destinations/UrlDestinationsLoading';
import UrlExtraStats from './UrlExtraStats';
import UrlStatsCount from './UrlStatsCount';
import UrlStatsCountry from './UrlStatsCountry';
import UrlStatsDailyChart from './UrlStatsDailyChart';
import UrlStatsDevice from './UrlStatsDevice';
import UrlStatsOrigin from './UrlStatsOrigin';

class UrlStatistics extends Component {
    render() {
        return (
            <React.Fragment>
                <div className={`hp-url-statistics ${this.isCompact() ? '--compact' : '--default'}`}>
                    {this.getLayout()}
                </div>
            </React.Fragment>
        );
    }

    getLayout() 
    {        
        if (!this.props.hasLoaded? !this.props.isRealoading : false) {
            return <LoadingOverlay show={true} />;//<UrlDestinationsLoading icon={(<InsertChartRounded />)} title={HighWayPro.text.analytics.loadingMessage}/>;
        }

        if (this.isCompact()) {
            return (
                <React.Fragment>
                    <div className="hp-row">
                        {this.getAnalyticsComponent('UrlStatsCount')}
                    </div>
                    <div className="hp-row">
                        <div className="hp-col hp-col--countries">
                            {this.getAnalyticsComponent('UrlStatsCountry')}
                        </div>
                        <div className="hp-col hp-col--wide">
                            {this.getAnalyticsComponent('UrlStatsDevice')}
                            {this.getAnalyticsComponent('UrlStatsOrigin')}
                        </div>
                    </div>
                </React.Fragment>
            )
        } else {
            return (
                <React.Fragment>
                    <div className="hp-row">
                        {this.getAnalyticsComponent('UrlStatsCount')}
                        {this.getAnalyticsComponent('UrlStatsCountry')}
                    </div>
                    <div className="hp-row">
                        {this.getAnalyticsComponent('UrlStatsDevice')}
                        {this.getAnalyticsComponent('UrlStatsOrigin')}
                    </div>
                </React.Fragment>
            )
        }
    }

    getAnalyticsComponent(name) 
    {
        const data = this.props.getData();

        switch (name) {
            case 'UrlStatsCount':
                return (<UrlStatsCount statistics={data.statistics} periodOfTime={this.props.periodOfTime} isRealoading={this.props.isRealoading}/>);
            break;
            case 'UrlStatsCountry':
                return (<UrlStatsCountry statistics={data.statistics} periodOfTime={this.props.periodOfTime} isRealoading={this.props.isRealoading}/>);
            break;
            case 'UrlStatsDevice':
                return (<UrlStatsDevice statistics={data.statistics} periodOfTime={this.props.periodOfTime} isRealoading={this.props.isRealoading}/>);
            break;
            case 'UrlStatsOrigin':
                return (<UrlStatsOrigin statistics={data.statistics} periodOfTime={this.props.periodOfTime} isRealoading={this.props.isRealoading}/>);
            break;
        }
    }
    isCompact() 
    {
        return this.props.layout === 'compact'
    }
}

UrlStatistics.getFormattedName = function(name)
{
    if (name === HighWayPro.analytics.NA_KEY) {
        return HighWayPro.text.analytics.NA;
    }

    name = name || '';

    // capitalize name except for urls, not bullet proof but useful for now, YAGNI.
    return `${name}`.trim().indexOf('http') !== 0? Strings.ucfirst(name) : name;
}

UrlStatistics.getShortFormattedName = function(name)
{
    if (name === HighWayPro.analytics.NA_KEY) {
        return HighWayPro.text.analytics.NA_SHORT;
    }

    return name;
}
export default UrlStatistics;