import './UrlStatsCount.css';

import NumberFormat from 'react-number-format';
import React, {Component} from 'react';
import Tooltip from '@material-ui/core/Tooltip';
import classNames from 'classnames';

import { HighWayPro } from '../../../../../domain/highwaypro/HighWayPro';
import { Strings } from '../../../../../domain/utilities/Strings';
import StatStateBar from './StatStateBar';
import UrlStatsDailyChart from './UrlStatsDailyChart';

class UrlStatsCount extends Component {

    classes = {
        current: '--hp-current'
    };

    render() {
        return (
            <div className="hp-url-stats-count-container hp-stats-box">
               {false && (
                    <div className="hp-box-heading">
                        <h1>{HighWayPro.text.analytics.clicksOverviewTitle}</h1>
                        <p>{HighWayPro.text.analytics.clicksOverviewMessage}</p>
                    </div>
                )}
                <header className="hp-top-presets">
                    {this.getData().map((stat, index) => (
                        <div key={stat.key} 
                            className={classNames({
                                'hp-url-stats-count': true,
                                [this.classes.current]: stat.key.toLowerCase() === this.props.periodOfTime.preset.toLowerCase(),
                                'hp-selectable': this.topPresetIsSelectable(stat.key)
                            })}
                            onClick={this.handleTopPresetClick.bind(this, stat.key)}
                        >
                            <div className="hp-stat-count-type">
                                {stat.label}
                            </div>
                            <div className="hp-stat-count-number">
                                <NumberFormat 
                                    value={stat.count.total} 
                                    displayType={'text'} 
                                    thousandSeparator={HighWayPro.preferences.analytics.thousandSeparator} 
                                    decimalSeparator={HighWayPro.preferences.analytics.decimalSeparator}
                                />
                            </div>
                            {typeof stat.count.difference !== 'undefined'? (
                                <div className={`hp-stat-difference ${this.getPercentageState(stat.count.difference.percentage)}`}>
                                     <Tooltip title={Strings.ucfirst(`${Math.abs(stat.count.difference.difference)} ${this.getDifferenceAsString(stat.count.difference.difference)} than ${HighWayPro.text.analytics.presetsSamePeriod[stat.key]}`.toLowerCase())} placement="top">
                                        <span>
                                            <NumberFormat 
                                                value={stat.count.difference.percentage} 
                                                displayType={'text'} 
                                                thousandSeparator={HighWayPro.preferences.analytics.thousandSeparator} 
                                                decimalSeparator={HighWayPro.preferences.analytics.decimalSeparator}
                                                prefix={this.percentageIsPositive(stat.count.difference.percentage) && '+' || ''} 
                                                decimalScale={2}
                                                suffix={'%'}
                                            />
                                        </span>
                                      </Tooltip>
                                </div> 
                            ) : ''}
                        </div>
                    ))}
                </header>
                <UrlStatsDailyChart periodCount={this.props.statistics.inPeriod.periodCount} isRealoading={this.props.isRealoading}/>
                <StatStateBar periodOfTime={this.props.periodOfTime} statistics={this.props.statistics}/>
            </div>
        );
    }

    handleTopPresetClick(preset) 
    {
        if (this.topPresetIsSelectable(preset)) {
            this.props.periodOfTime.setProperty({preset});
        }
    }

    topPresetIsSelectable(preset) 
    {
        return !this.presetIsSelected(preset) && 
                this.isAValidPreset(preset);
    }

    presetIsSelected(preset) 
    {
        return this.props.periodOfTime.preset.toLowerCase() === preset.toLowerCase();
    }

    isAValidPreset(preset) 
    {
        return HighWayPro.analytics.presets[preset];
    }

    getData() 
    {
        return this.getTopPresets().map(({preset, countItem}) => {
            if (typeof countItem !== 'object') {
                countItem = {
                    total: countItem
                };
            }

            return {
                label: HighWayPro.text.analytics.presets[preset] || HighWayPro.analytics.presets[preset] || '',
                key: preset.toLowerCase(),
                count: countItem
            }
        })
    }

    // what were going to do here is: 
    // if the user selected a preset that was not included in the default top presets
    // then we'll add it here with its title and its total count.
    getTopPresets() 
    {
        const topPresets = Object.keys(this.props.statistics.count);

        // let's move all time to the end
        const allTime = topPresets.splice(topPresets.findIndex(preset => preset.toLowerCase() === 'alltime'), 1);

        if (Array.isArray(allTime)) {
            topPresets.push(allTime[0]);
        }

        const topPresetItems = topPresets.map(topPreset => ({
            preset: topPreset,
            countItem: this.props.statistics.count[topPreset]
        }));

        if (!topPresets.map(topPreset => topPreset.toLowerCase()).includes(this.props.periodOfTime.preset.toLowerCase())) {
            topPresetItems.unshift({
                preset: this.props.periodOfTime.preset.toLowerCase(),
                countItem: this.props.statistics.inPeriod.periodCount.total
            });
        }

        return topPresetItems;
    }

    getDifferenceAsString(number) 
    {
        if (number < 0) {
            return 'less';
        } else {
            return 'more'
        }
    }

    percentageIsPositive(percentage) 
    {
        return percentage >= 0;
    }

    getPercentageState(percentage) 
    {
        if (percentage < 0) {
            return 'hpPercentageNegative';
        } else if (percentage == 0) {
            return 'hpPercentageNeutral';
        } else {
            return 'hpPercentagePositive';
        }
    }
}

export default UrlStatsCount;