import './UrlStatsCountry.css';

import React, {Component} from 'react';

import { HighWayPro } from '../../../../../domain/highwaypro/HighWayPro';
import BarChart from './BarChart';
import CustomPieChart from './CustomPieChart';
import IconWithText from '../../../../icons/IconWithText';
import LoadingOverlay from './LoadingOverlay';
import NotEnoughData from './NotEnoughData';
import ReadOnlyStateBar from './ReadOnlyStateBar';

class UrlStatsCountry extends Component {
    render() {
        const data = this.props.statistics.inPeriod.countByField.countries;

        if (!data || !data.length) {
            return <NotEnoughData statsType="country"/>
        }

        return (
            <div className="hp-url-stats-country-container hp-stats-box">
                <LoadingOverlay show={this.props.isRealoading}/>
                <div className="hp-box-heading">
                    <h1>{HighWayPro.text.analytics.countryClicksTitle}</h1>
                    <p>
                        <ReadOnlyStateBar 
                            periodOfTime={this.props.periodOfTime} 
                            date={this.props.statistics.inPeriod.periodCount.date}
                        />
                    </p>
                </div>
                <BarChart data={data} field={{name: 'location_country', value: 'percentage'}}/>
            </div>
        );
    }
}

export default UrlStatsCountry;