import './UrlStatsDailyChart.css';

import {
  Area,
  AreaChart,
  CartesianGrid,
  Tooltip,
  XAxis,
  YAxis
} from 'recharts';
import BarLoader from 'react-spinners/BarLoader';
import React, {Component} from 'react';
import _ from 'lodash';

import { month } from '../../../../../domain/utilities/dates/month';
import CustomChartToolTip from './CustomChartToolTip';
import LoadingOverlay from './LoadingOverlay';

class UrlStatsDailyChart extends Component {
    state = {
        width: 600 // initial width...
    };

    elements = {
        main: React.createRef()
    };

    getDate(offset) 
    {
        return this.props.periodCount.count.filter(day => {
            return day.offset == offset;
        })[0];
    }

    componentDidMount() 
    {
        this.setWidth();
        window.addEventListener('resize', this.setWidth.bind(this));
    }

    componentDidUpdate() 
    {
        this.setWidth();
    }

    render() {
        return (
            <div className="hp-url-stats-daily-chart" ref={this.elements.main}>
                <LoadingOverlay show={this.props.isRealoading} />
                <AreaChart width={this.state.width} height={300} data={this.props.periodCount.count}
                    margin={{top: 10, right: 0, left: 0, bottom: 0}}>
                    <CartesianGrid strokeDasharray="2 2"/>
                    <XAxis 
                        type="number"
                        domain={[this.props.periodCount.count.length, 1]}
                        dataKey="offset" 
                        ticks={this.getTicks()}
                        tickCount={this.getThickCount()}
                        minTickGap={1}
                        tickFormatter={offset => {
                            offset = Math.max(offset, 0);
                            const date = this.getDate(offset);

                            if (this.props.periodCount.unit === 'days') {
                                return  this.getdayAndMonthFormatted(date.dateRange.from);
                            } else if (this.props.periodCount.unit === 'hours') {
                                return date.labels.date;
                            } else if (this.props.periodCount.unit === 'months') {
                                return month(date.dateRange.from.month);
                            }

                            return `${this.getdayAndMonthFormatted(date.dateRange.from)} -> ${this.getdayAndMonthFormatted(date.dateRange.to)}`
                        }}
                    />
                    <YAxis 
                        dataKey="count" 
                        allowDecimals={false}
                    />
                    <Tooltip
                        content={({payload: data}) => {
                            let day = (data[0] && data[0].payload) || {};

                            return <CustomChartToolTip day={day || {}} />;
                        }}
                    />
                    <defs>
                        <linearGradient id="hp-gradient" spreadMethod="pad" x1="0%" y1="0%" x2="0%" y2="100%">
                        <stop offset="0%" style={{stopColor:'rgba(90, 150, 253, 0.99)',stopOpacity:'    0.99'}} />
                            <stop offset="37%" style={{stopColor:'rgba(90, 150, 253, 0.24)',stopOpacity:'0.24'}} />
                            <stop offset="48" style={{stopColor:'rgba(90, 150, 253, 0)',stopOpacity:'0'}} />
                        </linearGradient>
                        </defs>
                    <Area type='linear' dot={{fill: '#5a96fd', fillOpacity: 1, r: 3.12}} dataKey='count' stroke={'#5a96fd'} fill='url(#hp-gradient)' animationDuration={420}/>
              </AreaChart>
            </div>
        );
    }

    getLoadingAnimation() 
    {
        return (
            <div className="loadingOverlay">
                <BarLoader height={5} width={136} radius={6} color={'#368aff'}/>
            </div>
        );
    }

    getdayAndMonthFormatted(dateRange) 
    {
        return _.truncate(month(dateRange.month), {length: 3, omission: ''}) + ` ${dateRange.day}` || ''
    }

    getTicks() 
    {
        const ticks = [];
        const tickCount = this.getThickCount();
        const highestOffset = this.props.periodCount.count.length - 1;
        _.range(0, tickCount).map(i => {
            ticks.push(
                Math.min(i * this.getTickGap(), highestOffset)
            );
        });

        if (ticks[ticks.length - 1] !== highestOffset) {
            ticks.push(highestOffset);
        }

        return ticks;

    }

    getThickCount() 
    {
        return this.props.periodCount.count.length / this.getTickGap();
    }

    getTickGap() 
    {
        return this.props.periodCount.unit === 'days'? 7 : 1
    }

    setWidth() 
    {
        if (!this.elements.main.current) return;

        const newWidth = this.elements.main.current.offsetWidth - 30;

        if (newWidth != this.state.width) {
            this.setState({
                width: newWidth
            })
        }
    }
}

export default UrlStatsDailyChart;