import './UrlStatsDevice.css';

import React, {Component} from 'react';

import { HighWayPro } from '../../../../../domain/highwaypro/HighWayPro';
import BarChart from './BarChart';
import CustomPieChart from './CustomPieChart';
import LoadingOverlay from './LoadingOverlay';
import NotEnoughData from './NotEnoughData';
import ReadOnlyStateBar from './ReadOnlyStateBar';

class UrlStatsDevice extends Component {
    render() {
        const data = this.props.statistics.inPeriod.countByField.devices;

        if (!data || !data.length) {
            return <NotEnoughData statsType="device"/>
        }
        return (
            <div className="hp-url-stats-device-container hp-stats-box --withDivisions">
                <LoadingOverlay show={this.props.isRealoading} />
                <div className="hp-box-heading">
                    <h1>{HighWayPro.text.analytics.deviceClicksTitle}</h1>
                    <p>
                        <ReadOnlyStateBar 
                            periodOfTime={this.props.periodOfTime} 
                            date={this.props.statistics.inPeriod.periodCount.date}
                        />
                    </p>
                </div>
                <div className="stat-content">
                    <CustomPieChart data={data} field={{name: 'device_type'}} width={250} height={250}/>
                </div>
            </div>
        );
    }
}

export default UrlStatsDevice;