import './UrlStatsOrigin.css';

import React, {Component} from 'react';

import { HighWayPro } from '../../../../../domain/highwaypro/HighWayPro';
import BarChart from './BarChart';
import CustomPieChart from './CustomPieChart';
import LoadingOverlay from './LoadingOverlay';
import MultiBarChart from './MultiBarChart';
import NotEnoughData from './NotEnoughData';
import ReadOnlyStateBar from './ReadOnlyStateBar';

class UrlStatsOrigin extends Component {
    render() {
        const data = this.props.statistics.inPeriod.countByField.origin;

        if (!data || !data.length) {
            return <NotEnoughData statsType="origin" />
        }
        return (
            <div className="hp-url-stats-origin-container hp-stats-box --withDivisions">
                <LoadingOverlay show={this.props.isRealoading} />
                <div className="hp-box-heading">
                    <h1>{HighWayPro.text.analytics.originClicksTitle}</h1>
                    <p>
                        <ReadOnlyStateBar 
                            periodOfTime={this.props.periodOfTime} 
                            date={this.props.statistics.inPeriod.periodCount.date}
                        />
                    </p>
                </div>
                <div className="stat-content">
                    <CustomPieChart data={data} 
                                    field={{name: 'device_referer'}} 
                                    innerRadius={1}
                                    width={250} 
                                    height={250}
                                    colors={['#bbbc3f', '#00b0f4', '#1368bb', '#31c5b9', '#ffcd26', '#b6d0e0']}
                    />
                </div>
            </div>
        );
    }
}

export default UrlStatsOrigin;