import { HighWayPro } from '../highwaypro/HighWayPro';
import { Observable } from '../behaviour/events/observer/Observable';
import format from 'date-fns/format';

export default class PeriodOfTime extends Observable
{
    preset = HighWayPro.preferences.analytics.defaultPreset;
    unit = HighWayPro.preferences.analytics.defaultUnit;
    customRange = {
        from: '',
        to: ''
    };

    constructor(preset, unit) 
    {
        super();

        this.preset = preset || this.preset;
        this.unit = unit || this.unit;
    }

    /*
        Properties must me set using this method, otherwise observes wont be notified.
     */
    setProperty({preset, unit, customRange}) 
    {
        if ((unit || this.unit) === 'hours' && (preset || this.preset) !== 'today') {
            unit = 'auto';
        }

        this.update(periodOfTime => {
            periodOfTime.preset = preset || periodOfTime.preset;
            periodOfTime.unit = unit || periodOfTime.unit;
            periodOfTime.customRange = customRange || periodOfTime.customRange;
        });
    }

    getReadablePreset(optionalPreset, includeCustomRangeLabel) 
    {
        const preset = optionalPreset || this.preset;

        if (preset.toLowerCase() === 'customrange' && includeCustomRangeLabel == false) return;
        
        return HighWayPro.analytics.presets[preset] || preset;
    }

    getReadableRange(optionalRange, type) 
    {
        type = type || 'normal';

        const range = optionalRange || this.customRange;

        let readableformat = type === 'short'? "LLL do ''yy":  'EEE, LLL do yyyy';

        if (format(range.from, 'YYYY-MM-dd') === format(range.to, 'YYYY-MM-dd')) {
            // its the same day, just show the day
            return format(range.from, readableformat);
        } else {
            return `${format(range.from, readableformat)} -> ${format(range.to, readableformat)}`;
        }   
    }
}
