import AccessTimeRounded from '@material-ui/icons/AccessTimeRounded';
import React from 'react';
import FormControl from '@material-ui/core/FormControl';
import InputLabel from '@material-ui/core/InputLabel';
import MenuItem from '@material-ui/core/MenuItem';
import Select from '@material-ui/core/Select';

import { BuiltInComponent } from '../BuiltInComponent';
import _ from 'lodash';

export class HourRangeCondition extends BuiltInComponent {
    static type = 'condition';
    static getIdStatic() { return 'highwaypro.HourRangeCondition';};
    getId() { return HourRangeCondition.getIdStatic() };
    icon = (<AccessTimeRounded />);

    initialState = {
        dayHours: {
            start: this.data.parameters.dayHours.start || 0,
            end: this.data.parameters.dayHours.end || 0
        }
    };

    static rangeTypes = [
        {
            label: 'From',
            property: 'start'
        },
        {
            label: 'To',
            property: 'end'
        }
    ];

    handleChange = property => ({target}) => {
        this.reactComponent.setState(({dayHours}) => ({
            dayHours: Object.assign(dayHours, {
                [property]: target.value
            })
        }));
    }

    getContent() 
    {
        return (
            <div id="hp-HourRangeCondition">
                {HourRangeCondition.rangeTypes.map(({label, property}) => (
                    <FormControl classes={{root: 'MuiFormControl-root'}}>
                      <InputLabel>{label}</InputLabel>
                      <Select classes={{root:'MuiInputBase-input MuiInput-underline'}}
                        className="MuiInput-underline MuiInputBase-input"
                        value={this.reactComponent.state.dayHours[property]}
                        onChange={this.handleChange(property)}
                      >
                        {_.range(0, 24).map(hour => (
                          <MenuItem
                            key={hour}
                            value={hour}
                          >
                            {this.getDisplayValueFor(hour)}
                          </MenuItem>
                        ))}
                      </Select>
                    </FormControl>
                ))}
            </div>
        );
    }

    getDisplayValueFor(hour) 
    {
        const timeOfDay = hour < 12? 'a.m' : 'p.m';

        hour = hour == 0? '12' : hour;
        hour = hour > 12? hour - 12 : hour;

        return `${hour} ${timeOfDay}`
    }

    getValuesAsPreview()
    {
        if (this.data.parameters.dayHours.start !== this.data.parameters.dayHours.end) {
            return (
                <div>
                     <span className="ligh-contrast">From</span> <b>{this.getDisplayValueFor(this.data.parameters.dayHours.start)}</b> <span className="ligh-contrast">to</span> <b>{this.getDisplayValueFor(this.data.parameters.dayHours.end)}</b>                    
                </div>
            )
        } 

        return (
            <div><b>{this.getDisplayValueFor(this.data.parameters.dayHours.start)}</b> <span class="ligh-contrast">only</span></div>
        );
    }
}