import { isObject } from '../../../utilities/isObject';

import AsyncSelect from 'react-select/lib/Async';
import FormControl from '@material-ui/core/FormControl';
import InputLabel from '@material-ui/core/InputLabel';
import VerticalSplitRounded from '@material-ui/icons/VerticalSplitRounded';
import MenuItem from '@material-ui/core/MenuItem';
import React from 'react';
import Select from '@material-ui/core/Select';
import classnames from 'classnames';

import { BuiltInComponent } from '../BuiltInComponent';
import { PostsFinder } from '../../../data/finders/PostsFinder';

export class ContentTarget extends BuiltInComponent {
    static type = 'target';
    static getIdStatic() { return 'highwaypro.ContentTarget';}
    getId() { return ContentTarget.getIdStatic() } 
    icon = (<VerticalSplitRounded />);

    initialState = {
        id: this.data.parameters.id || 0,
    }

    constructor(data)
    {
        super(data);

        this.postsFinder = new PostsFinder;
    }

    handleChange = type => event => {
        this.reactComponent.setState({
            [type]: event.target.value
        });
    }

    loadPosts = input => {
        return this.postsFinder.getByKeyword(input).then(posts => posts.map(post => {
            return { value: post.ID, label: post.post_title }
        }));
    }

    handlePostIdChange = (selected) => {
        this.selectedPost = isObject(selected)? selected : {};

        this.reactComponent.setState({
            id: selected.value
        })
    }

    getPlaceHolder() 
    {
        if (this.data.parameters.id > 0) {
            return `Post Id: ${this.data.parameters.id}`;
        }

        return 'Search Content';
    }
    getContent() 
    {
        return (
            <React.Fragment>
            <AsyncSelect 
                className={classnames({'hp-select-search': true, '--disabled': false})} 
                classNamePrefix="hp-select-search" 
                placeholder={this.getPlaceHolder()} 
                onChange={this.handlePostIdChange} 
                loadOptions={this.loadPosts} 
                noOptionsMessage={() => 'Search content to continue'}
            />
            </React.Fragment>
        )
    }

    getValuesAsPreview()
    {
        return `Content with id: ${this.data.parameters.id}`;       
    }
}